-- ============================================================= --
-- LUMBERJACK MOD
-- ============================================================= --
LumberJack = {};
source(g_currentModDirectory .. 'DeleteShapeEvent.lua');

addModEventListener(LumberJack);

-- ALLOW CHAINSAW CUTTING ANYWHERE ON THE MAP
function LumberJack:isCuttingAllowed(superFunc, x, y, z)
	return true;
end

function LumberJack:loadMap(name)
	--print("Load Mod: 'LumberJack'")
	self.setOnceFlag = false
	self.useChainsawFlag = false
	self.foundObjectFlag = false
	self.pickUpObjectFlag = false
	self.driveVehicleFlag = false
	self.hasHandtoolEquipped = false
	self.altKeyReleased = true
	self.superStrength = false
	self.lastObjectMass = 0
	self.superStrengthValue = 100
	self.normalStrengthValue = 0.2
	self.ringSelector = 0
	
	-- ALLOW CHAINSAW CUTTING ANYWHERE ON THE MAP
	Chainsaw.isCuttingAllowed = Utils.overwrittenFunction(Chainsaw.isCuttingAllowed, LumberJack.isCuttingAllowed);
end
function LumberJack:deleteMap()
end
function LumberJack:mouseEvent(posX, posY, isDown, isUp, button)
end
function LumberJack:keyEvent(unicode, sym, modifier, isDown)
	if not (self.pickUpObjectFlag or self.driveVehicleFlag or self.hasHandtoolEquipped) then
	-- TOGGLE SUPER STRENGTH ON/OFF WITH ALT KEY
		if bitAND(modifier, Input.MOD_ALT) > 0 then
			if altKeyReleased then			
				if self.superStrength then
					--print("SUPER STRENGTH OFF")
					Player.MAX_PICKABLE_OBJECT_MASS = self.normalStrengthValue
					self.superStrength = false
					altKeyReleased = false
				else
					--print("SUPER STRENGTH ON")
					Player.MAX_PICKABLE_OBJECT_MASS = self.superStrengthValue
					self.superStrength = true
					altKeyReleased = false
				end
			end
		else
			altKeyReleased = true
		end
--[[		
	-- HOLD DOWN ALT KEY FOR SUPER STRENGTH	
		if self.superStrength then
			--print("SUPER STRENGTH OFF")
			Player.MAX_PICKABLE_OBJECT_MASS = self.normalStrengthValue
			self.superStrength = false
		else
			if bitAND(modifier, Input.MOD_ALT) > 0 then
				--print("SUPER STRENGTH ON")
				Player.MAX_PICKABLE_OBJECT_MASS = self.superStrengthValue
				self.superStrength = true
			end
		end
]]--
	end
end
function LumberJack:draw()
end

function LumberJack:update(dt)
	-- CHANGE GLOBAL VALUES ON FIRST RUN
	if (g_gameStateManager:getGameState()==GameState.PLAY and self.setOnceFlag==false) then
		-- Only change values from default
		if Player.MAX_PICKABLE_OBJECT_DISTANCE == 3.00 then
			Player.MAX_PICKABLE_OBJECT_DISTANCE = 6.00
		end
		if g_currentMission.player.minCutDistance == 0.50 then
			g_currentMission.player.minCutDistance = 0.10
		end
		if g_currentMission.player.maxCutDistance == 2.00 then
			g_currentMission.player.maxCutDistance = 4.00
		end
		self.setOnceFlag = true
	end

	-- IF ENTERING A VEHICLE
	if g_currentMission.controlledVehicle then
		if not self.driveVehicleFlag then
			--print("PLAYER IS DRIVING")
			if self.superStrength then
				--print("SUPER STRENGTH OFF")
				Player.MAX_PICKABLE_OBJECT_MASS = self.normalStrengthValue
				self.superStrength = false
			end
			self.driveVehicleFlag = true
		end
	else
		self.driveVehicleFlag = false
	end
	
	-- IF OBSERVING AN OBJECT
	if g_currentMission.player.isObjectInRange then
		-- check if we saw this object last time
		if self.lastObjectMass ~= g_currentMission.player.lastFoundObjectMass then
			self.lastObjectMass = g_currentMission.player.lastFoundObjectMass
		end

		-- Display Mass of LAST OBSERVED OBJECT in 'F1' Help Menu
		if not (self.driveVehicleFlag or g_currentMission.player:hasHandtoolEquipped()) then
			g_currentMission:addExtraPrintText(string.format("Mass: %.1f kg", 1000*self.lastObjectMass))
		end

		self.foundObjectFlag = true
	else
		self.lastObjectMass = 0
		self.foundObjectFlag = false
	end

	-- IF ACTUALLY CARRYING AN OBJECT
	if g_currentMission.player.isCarryingObject then
		if not self.pickUpObjectFlag then
			-- adjust super strength when picking up an object
			if self.superStrength and 5*self.lastObjectMass > self.normalStrengthValue then
				Player.MAX_PICKABLE_OBJECT_MASS = 5*self.lastObjectMass
			end
			self.pickUpObjectFlag = true
		end
	else
	-- IF NOT CARRYING AN OBJECT
		if self.pickUpObjectFlag then
			-- remove super strength when dropping an object
			--self.superStrength = false
			--Player.MAX_PICKABLE_OBJECT_MASS = self.normalStrengthValue
			
			-- Restore Full super strength when dropping an object
			if self.superStrength then
				Player.MAX_PICKABLE_OBJECT_MASS = self.superStrengthValue
			end
			
			self.pickUpObjectFlag = false
		end
	end
	
	-- CHANGE COLOUR OF THE CURSOR/HAND ICON
	if self.superStrength then -- Make hand BRIGHTER when super strength is ON
		g_currentMission.player.pickedUpObjectOverlay:setColor(1, 1, 1, 1.0)
	elseif self.lastObjectMass > Player.MAX_PICKABLE_OBJECT_MASS then
		-- Make hand RED when objects are too heavy to pick up
		g_currentMission.player.pickedUpObjectOverlay:setColor(1.0, 0.1, 0.1, 0.5)
	else -- Make hand GREY for everything else
		g_currentMission.player.pickedUpObjectOverlay:setColor(1, 1, 1, 0.3)
	end
	
	-- DESTROY SMALL LOGS WHEN USING THE CHAINSAW --
	if g_currentMission.player:hasHandtoolEquipped() then
		self.hasHandtoolEquipped = true
		local hTool = g_currentMission.player.baseInformation.currentHandtool
		
		if hTool ~= nil then
			-- INCRESE CUTTING SPEED x2 (default value is 8.0)
			hTool.defaultCutDuration = 4.0
			
			if hTool.isCutting then
				--print("CHAINSAW CUTTING")				
				if self.ringSelector ~= 0 then
					setVisibility(self.ringSelector, false)
				end
				if not self.useChainsawFlag then
					-- Find the splitShape from chainsawSplitShapeFocus (lastFoundObject doesn't exist for client in multiplayer)
					local x,y,z = getWorldTranslation(g_currentMission.player.chainsawSplitShapeFocus)
					local nx,ny,nz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 1,0,0)
					local yx,yy,yz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,1,0)
					local splitShape, minY, maxY, minZ, maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, 5, 5)
					if splitShape ~=0 then
						if getVolume(splitShape) < 0.100 then
						-- DELETE THE SHAPE if too small to worry about (e.g. felling wedge or thin branch)
							if g_currentMission.missionDynamicInfo.isMultiplayer then
							--print("MULTIPLAYER")
								DeleteShapeEvent.sendEvent(splitShape)
							else
							--print("SINGLE PLAYER")
								delete(splitShape)
							end
						end
					end
					self.useChainsawFlag = true
				end
			else
				--print("CHAINSAW NOT CUTTING")
				if self.ringSelector == 0 then
					local filename = 'data/firstPerson/chainsaws/chainsawRingSelector.i3d'
					local i3dNode = g_i3DManager:loadSharedI3DFile(filename, g_currentMission.baseDirectory, false, false, false)
					if i3dNode ~= 0 then
						self.ringSelector = getChildAt(i3dNode, 0)
						self.ringSelectorScaleOffset = 0.0
						setVisibility(self.ringSelector, true)
						setShaderParameter(self.ringSelector, "colorScale", 0.05, 0.05, 0.05, 1.0, false)
						link(g_currentMission.player.chainsawCameraFocus, self.ringSelector)
						delete(i3dNode)
					end
				else
					if hTool.ringSelector ~= nil and hTool.ringSelector ~= 0 then
						if getVisibility(hTool.ringSelector) then
							setVisibility(self.ringSelector, false)
						else
							setVisibility(self.ringSelector, true)
						end
					end
				end
				
				self.useChainsawFlag = false
			end
		end
	else
		self.hasHandtoolEquipped = false
		if self.ringSelector ~= 0 then
			if getVisibility(self.ringSelector) then
				setVisibility(self.ringSelector, false)
			end
		end
	end
end